<?php

namespace ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\Input;

use ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Exception\InvalidArgument;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Input;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Request;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Stream\StreamFactory;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\Enum\ChecksumAlgorithm;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\Enum\ObjectCannedACL;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\Enum\RequestPayer;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\ValueObject\AccessControlPolicy;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\ValueObject\Grantee;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\ValueObject\Owner;
final class PutObjectAclRequest extends \ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Input
{
    /**
     * The canned ACL to apply to the object. For more information, see Canned ACL.
     *
     * @see https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#CannedACL
     *
     * @var ObjectCannedACL::*|null
     */
    private $acl;
    /**
     * Contains the elements that set the ACL permissions for an object per grantee.
     *
     * @var AccessControlPolicy|null
     */
    private $accessControlPolicy;
    /**
     * The bucket name that contains the object to which you want to attach the ACL.
     *
     * @required
     *
     * @var string|null
     */
    private $bucket;
    /**
     * The base64-encoded 128-bit MD5 digest of the data. This header must be used as a message integrity check to verify
     * that the request body was not corrupted in transit. For more information, go to RFC 1864.&gt;.
     *
     * @see http://www.ietf.org/rfc/rfc1864.txt
     *
     * @var string|null
     */
    private $contentMd5;
    /**
     * Indicates the algorithm used to create the checksum for the object when using the SDK. This header will not provide
     * any additional functionality if not using the SDK. When sending this header, there must be a corresponding
     * `x-amz-checksum` or `x-amz-trailer` header sent. Otherwise, Amazon S3 fails the request with the HTTP status code
     * `400 Bad Request`. For more information, see Checking object integrity in the *Amazon S3 User Guide*.
     *
     * @see https://docs.aws.amazon.com/AmazonS3/latest/userguide/checking-object-integrity.html
     *
     * @var ChecksumAlgorithm::*|null
     */
    private $checksumAlgorithm;
    /**
     * Allows grantee the read, write, read ACP, and write ACP permissions on the bucket.
     *
     * @var string|null
     */
    private $grantFullControl;
    /**
     * Allows grantee to list the objects in the bucket.
     *
     * @var string|null
     */
    private $grantRead;
    /**
     * Allows grantee to read the bucket ACL.
     *
     * @var string|null
     */
    private $grantReadAcp;
    /**
     * Allows grantee to create new objects in the bucket.
     *
     * @var string|null
     */
    private $grantWrite;
    /**
     * Allows grantee to write the ACL for the applicable bucket.
     *
     * @var string|null
     */
    private $grantWriteAcp;
    /**
     * Key for which the PUT action was initiated.
     *
     * @required
     *
     * @var string|null
     */
    private $key;
    /**
     * @var RequestPayer::*|null
     */
    private $requestPayer;
    /**
     * VersionId used to reference a specific version of the object.
     *
     * @var string|null
     */
    private $versionId;
    /**
     * The account ID of the expected bucket owner. If the bucket is owned by a different account, the request fails with
     * the HTTP status code `403 Forbidden` (access denied).
     *
     * @var string|null
     */
    private $expectedBucketOwner;
    /**
     * @param array{
     *   ACL?: ObjectCannedACL::*,
     *   AccessControlPolicy?: AccessControlPolicy|array,
     *   Bucket?: string,
     *   ContentMD5?: string,
     *   ChecksumAlgorithm?: ChecksumAlgorithm::*,
     *   GrantFullControl?: string,
     *   GrantRead?: string,
     *   GrantReadACP?: string,
     *   GrantWrite?: string,
     *   GrantWriteACP?: string,
     *   Key?: string,
     *   RequestPayer?: RequestPayer::*,
     *   VersionId?: string,
     *   ExpectedBucketOwner?: string,
     *   @region?: string,
     * } $input
     */
    public function __construct(array $input = [])
    {
        $this->acl = $input['ACL'] ?? null;
        $this->accessControlPolicy = isset($input['AccessControlPolicy']) ? \ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\ValueObject\AccessControlPolicy::create($input['AccessControlPolicy']) : null;
        $this->bucket = $input['Bucket'] ?? null;
        $this->contentMd5 = $input['ContentMD5'] ?? null;
        $this->checksumAlgorithm = $input['ChecksumAlgorithm'] ?? null;
        $this->grantFullControl = $input['GrantFullControl'] ?? null;
        $this->grantRead = $input['GrantRead'] ?? null;
        $this->grantReadAcp = $input['GrantReadACP'] ?? null;
        $this->grantWrite = $input['GrantWrite'] ?? null;
        $this->grantWriteAcp = $input['GrantWriteACP'] ?? null;
        $this->key = $input['Key'] ?? null;
        $this->requestPayer = $input['RequestPayer'] ?? null;
        $this->versionId = $input['VersionId'] ?? null;
        $this->expectedBucketOwner = $input['ExpectedBucketOwner'] ?? null;
        parent::__construct($input);
    }
    public static function create($input) : self
    {
        return $input instanceof self ? $input : new self($input);
    }
    public function getAccessControlPolicy() : ?\ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\ValueObject\AccessControlPolicy
    {
        return $this->accessControlPolicy;
    }
    /**
     * @return ObjectCannedACL::*|null
     */
    public function getAcl() : ?string
    {
        return $this->acl;
    }
    public function getBucket() : ?string
    {
        return $this->bucket;
    }
    /**
     * @return ChecksumAlgorithm::*|null
     */
    public function getChecksumAlgorithm() : ?string
    {
        return $this->checksumAlgorithm;
    }
    public function getContentMd5() : ?string
    {
        return $this->contentMd5;
    }
    public function getExpectedBucketOwner() : ?string
    {
        return $this->expectedBucketOwner;
    }
    public function getGrantFullControl() : ?string
    {
        return $this->grantFullControl;
    }
    public function getGrantRead() : ?string
    {
        return $this->grantRead;
    }
    public function getGrantReadAcp() : ?string
    {
        return $this->grantReadAcp;
    }
    public function getGrantWrite() : ?string
    {
        return $this->grantWrite;
    }
    public function getGrantWriteAcp() : ?string
    {
        return $this->grantWriteAcp;
    }
    public function getKey() : ?string
    {
        return $this->key;
    }
    /**
     * @return RequestPayer::*|null
     */
    public function getRequestPayer() : ?string
    {
        return $this->requestPayer;
    }
    public function getVersionId() : ?string
    {
        return $this->versionId;
    }
    /**
     * @internal
     */
    public function request() : \ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Request
    {
        // Prepare headers
        $headers = ['content-type' => 'application/xml'];
        if (null !== $this->acl) {
            if (!\ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\Enum\ObjectCannedACL::exists($this->acl)) {
                throw new \ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Exception\InvalidArgument(\sprintf('Invalid parameter "ACL" for "%s". The value "%s" is not a valid "ObjectCannedACL".', __CLASS__, $this->acl));
            }
            $headers['x-amz-acl'] = $this->acl;
        }
        if (null !== $this->contentMd5) {
            $headers['Content-MD5'] = $this->contentMd5;
        }
        if (null !== $this->checksumAlgorithm) {
            if (!\ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\Enum\ChecksumAlgorithm::exists($this->checksumAlgorithm)) {
                throw new \ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Exception\InvalidArgument(\sprintf('Invalid parameter "ChecksumAlgorithm" for "%s". The value "%s" is not a valid "ChecksumAlgorithm".', __CLASS__, $this->checksumAlgorithm));
            }
            $headers['x-amz-sdk-checksum-algorithm'] = $this->checksumAlgorithm;
        }
        if (null !== $this->grantFullControl) {
            $headers['x-amz-grant-full-control'] = $this->grantFullControl;
        }
        if (null !== $this->grantRead) {
            $headers['x-amz-grant-read'] = $this->grantRead;
        }
        if (null !== $this->grantReadAcp) {
            $headers['x-amz-grant-read-acp'] = $this->grantReadAcp;
        }
        if (null !== $this->grantWrite) {
            $headers['x-amz-grant-write'] = $this->grantWrite;
        }
        if (null !== $this->grantWriteAcp) {
            $headers['x-amz-grant-write-acp'] = $this->grantWriteAcp;
        }
        if (null !== $this->requestPayer) {
            if (!\ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\Enum\RequestPayer::exists($this->requestPayer)) {
                throw new \ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Exception\InvalidArgument(\sprintf('Invalid parameter "RequestPayer" for "%s". The value "%s" is not a valid "RequestPayer".', __CLASS__, $this->requestPayer));
            }
            $headers['x-amz-request-payer'] = $this->requestPayer;
        }
        if (null !== $this->expectedBucketOwner) {
            $headers['x-amz-expected-bucket-owner'] = $this->expectedBucketOwner;
        }
        // Prepare query
        $query = [];
        if (null !== $this->versionId) {
            $query['versionId'] = $this->versionId;
        }
        // Prepare URI
        $uri = [];
        if (null === ($v = $this->bucket)) {
            throw new \ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Exception\InvalidArgument(\sprintf('Missing parameter "Bucket" for "%s". The value cannot be null.', __CLASS__));
        }
        $uri['Bucket'] = $v;
        if (null === ($v = $this->key)) {
            throw new \ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Exception\InvalidArgument(\sprintf('Missing parameter "Key" for "%s". The value cannot be null.', __CLASS__));
        }
        $uri['Key'] = $v;
        $uriString = '/' . \rawurlencode($uri['Bucket']) . '/' . \str_replace('%2F', '/', \rawurlencode($uri['Key'])) . '?acl';
        // Prepare Body
        $document = new \DOMDocument('1.0', 'UTF-8');
        $document->formatOutput = \false;
        $this->requestBody($document, $document);
        $body = $document->hasChildNodes() ? $document->saveXML() : '';
        // Return the Request
        return new \ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Request('PUT', $uriString, $query, $headers, \ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Stream\StreamFactory::create($body));
    }
    public function setAccessControlPolicy(?\ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\ValueObject\AccessControlPolicy $value) : self
    {
        $this->accessControlPolicy = $value;
        return $this;
    }
    /**
     * @param ObjectCannedACL::*|null $value
     */
    public function setAcl(?string $value) : self
    {
        $this->acl = $value;
        return $this;
    }
    public function setBucket(?string $value) : self
    {
        $this->bucket = $value;
        return $this;
    }
    /**
     * @param ChecksumAlgorithm::*|null $value
     */
    public function setChecksumAlgorithm(?string $value) : self
    {
        $this->checksumAlgorithm = $value;
        return $this;
    }
    public function setContentMd5(?string $value) : self
    {
        $this->contentMd5 = $value;
        return $this;
    }
    public function setExpectedBucketOwner(?string $value) : self
    {
        $this->expectedBucketOwner = $value;
        return $this;
    }
    public function setGrantFullControl(?string $value) : self
    {
        $this->grantFullControl = $value;
        return $this;
    }
    public function setGrantRead(?string $value) : self
    {
        $this->grantRead = $value;
        return $this;
    }
    public function setGrantReadAcp(?string $value) : self
    {
        $this->grantReadAcp = $value;
        return $this;
    }
    public function setGrantWrite(?string $value) : self
    {
        $this->grantWrite = $value;
        return $this;
    }
    public function setGrantWriteAcp(?string $value) : self
    {
        $this->grantWriteAcp = $value;
        return $this;
    }
    public function setKey(?string $value) : self
    {
        $this->key = $value;
        return $this;
    }
    /**
     * @param RequestPayer::*|null $value
     */
    public function setRequestPayer(?string $value) : self
    {
        $this->requestPayer = $value;
        return $this;
    }
    public function setVersionId(?string $value) : self
    {
        $this->versionId = $value;
        return $this;
    }
    private function requestBody(\DOMNode $node, \DOMDocument $document) : void
    {
        if (null !== ($v = $this->accessControlPolicy)) {
            $node->appendChild($child = $document->createElement('AccessControlPolicy'));
            $child->setAttribute('xmlns', 'http://s3.amazonaws.com/doc/2006-03-01/');
            $v->requestBody($child, $document);
        }
    }
}
