<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace ZOOlanders\YOOessentials\Vendor\Symfony\Component\HttpClient;

use ZOOlanders\YOOessentials\Vendor\GuzzleHttp\Promise\Promise as GuzzlePromise;
use ZOOlanders\YOOessentials\Vendor\GuzzleHttp\Promise\RejectedPromise;
use ZOOlanders\YOOessentials\Vendor\GuzzleHttp\Promise\Utils;
use ZOOlanders\YOOessentials\Vendor\Http\Client\Exception\NetworkException;
use ZOOlanders\YOOessentials\Vendor\Http\Client\Exception\RequestException;
use ZOOlanders\YOOessentials\Vendor\Http\Client\HttpAsyncClient;
use ZOOlanders\YOOessentials\Vendor\Http\Client\HttpClient as HttplugInterface;
use ZOOlanders\YOOessentials\Vendor\Http\Discovery\Exception\NotFoundException;
use ZOOlanders\YOOessentials\Vendor\Http\Discovery\Psr17FactoryDiscovery;
use ZOOlanders\YOOessentials\Vendor\Http\Message\RequestFactory;
use ZOOlanders\YOOessentials\Vendor\Http\Message\StreamFactory;
use ZOOlanders\YOOessentials\Vendor\Http\Message\UriFactory;
use ZOOlanders\YOOessentials\Vendor\Http\Promise\Promise;
use ZOOlanders\YOOessentials\Vendor\Nyholm\Psr7\Factory\Psr17Factory;
use ZOOlanders\YOOessentials\Vendor\Nyholm\Psr7\Request;
use ZOOlanders\YOOessentials\Vendor\Nyholm\Psr7\Uri;
use ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\RequestFactoryInterface;
use ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\RequestInterface;
use ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\ResponseFactoryInterface;
use ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\ResponseInterface as Psr7ResponseInterface;
use ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\StreamFactoryInterface;
use ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\StreamInterface;
use ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\UriFactoryInterface;
use ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\UriInterface;
use ZOOlanders\YOOessentials\Vendor\Symfony\Component\HttpClient\Internal\HttplugWaitLoop;
use ZOOlanders\YOOessentials\Vendor\Symfony\Component\HttpClient\Response\HttplugPromise;
use ZOOlanders\YOOessentials\Vendor\Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use ZOOlanders\YOOessentials\Vendor\Symfony\Contracts\HttpClient\HttpClientInterface;
use ZOOlanders\YOOessentials\Vendor\Symfony\Contracts\HttpClient\ResponseInterface;
use ZOOlanders\YOOessentials\Vendor\Symfony\Contracts\Service\ResetInterface;
if (!\interface_exists(\ZOOlanders\YOOessentials\Vendor\Http\Client\HttpClient::class)) {
    throw new \LogicException('You cannot use "Symfony\\Component\\HttpClient\\HttplugClient" as the "php-http/httplug" package is not installed. Try running "composer require php-http/httplug".');
}
if (!\interface_exists(\ZOOlanders\YOOessentials\Vendor\Http\Message\RequestFactory::class)) {
    throw new \LogicException('You cannot use "Symfony\\Component\\HttpClient\\HttplugClient" as the "php-http/message-factory" package is not installed. Try running "composer require nyholm/psr7".');
}
/**
 * An adapter to turn a Symfony HttpClientInterface into an Httplug client.
 *
 * Run "composer require nyholm/psr7" to install an efficient implementation of response
 * and stream factories with flex-provided autowiring aliases.
 *
 * @author Nicolas Grekas <p@tchwork.com>
 */
final class HttplugClient implements \ZOOlanders\YOOessentials\Vendor\Http\Client\HttpClient, \ZOOlanders\YOOessentials\Vendor\Http\Client\HttpAsyncClient, \ZOOlanders\YOOessentials\Vendor\Http\Message\RequestFactory, \ZOOlanders\YOOessentials\Vendor\Http\Message\StreamFactory, \ZOOlanders\YOOessentials\Vendor\Http\Message\UriFactory, \ZOOlanders\YOOessentials\Vendor\Symfony\Contracts\Service\ResetInterface
{
    private $client;
    private $responseFactory;
    private $streamFactory;
    /**
     * @var \SplObjectStorage<ResponseInterface, array{RequestInterface, Promise}>|null
     */
    private $promisePool;
    private $waitLoop;
    public function __construct(\ZOOlanders\YOOessentials\Vendor\Symfony\Contracts\HttpClient\HttpClientInterface $client = null, \ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\ResponseFactoryInterface $responseFactory = null, \ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\StreamFactoryInterface $streamFactory = null)
    {
        $this->client = $client ?? \ZOOlanders\YOOessentials\Vendor\Symfony\Component\HttpClient\HttpClient::create();
        $this->responseFactory = $responseFactory;
        $this->streamFactory = $streamFactory ?? ($responseFactory instanceof \ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\StreamFactoryInterface ? $responseFactory : null);
        $this->promisePool = \class_exists(\ZOOlanders\YOOessentials\Vendor\GuzzleHttp\Promise\Utils::class) ? new \SplObjectStorage() : null;
        if (null === $this->responseFactory || null === $this->streamFactory) {
            if (!\class_exists(\ZOOlanders\YOOessentials\Vendor\Nyholm\Psr7\Factory\Psr17Factory::class) && !\class_exists(\ZOOlanders\YOOessentials\Vendor\Http\Discovery\Psr17FactoryDiscovery::class)) {
                throw new \LogicException('You cannot use the "Symfony\\Component\\HttpClient\\HttplugClient" as no PSR-17 factories have been provided. Try running "composer require nyholm/psr7".');
            }
            try {
                $psr17Factory = \class_exists(\ZOOlanders\YOOessentials\Vendor\Nyholm\Psr7\Factory\Psr17Factory::class, \false) ? new \ZOOlanders\YOOessentials\Vendor\Nyholm\Psr7\Factory\Psr17Factory() : null;
                $this->responseFactory = $this->responseFactory ?? $psr17Factory ?? \ZOOlanders\YOOessentials\Vendor\Http\Discovery\Psr17FactoryDiscovery::findResponseFactory();
                $this->streamFactory = $this->streamFactory ?? $psr17Factory ?? \ZOOlanders\YOOessentials\Vendor\Http\Discovery\Psr17FactoryDiscovery::findStreamFactory();
            } catch (\ZOOlanders\YOOessentials\Vendor\Http\Discovery\Exception\NotFoundException $e) {
                throw new \LogicException('You cannot use the "Symfony\\Component\\HttpClient\\HttplugClient" as no PSR-17 factories have been found. Try running "composer require nyholm/psr7".', 0, $e);
            }
        }
        $this->waitLoop = new \ZOOlanders\YOOessentials\Vendor\Symfony\Component\HttpClient\Internal\HttplugWaitLoop($this->client, $this->promisePool, $this->responseFactory, $this->streamFactory);
    }
    /**
     * {@inheritdoc}
     */
    public function sendRequest(\ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\RequestInterface $request) : \ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\ResponseInterface
    {
        try {
            return $this->waitLoop->createPsr7Response($this->sendPsr7Request($request));
        } catch (\ZOOlanders\YOOessentials\Vendor\Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface $e) {
            throw new \ZOOlanders\YOOessentials\Vendor\Http\Client\Exception\NetworkException($e->getMessage(), $request, $e);
        }
    }
    /**
     * {@inheritdoc}
     *
     * @return HttplugPromise
     */
    public function sendAsyncRequest(\ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\RequestInterface $request) : \ZOOlanders\YOOessentials\Vendor\Http\Promise\Promise
    {
        if (!($promisePool = $this->promisePool)) {
            throw new \LogicException(\sprintf('You cannot use "%s()" as the "guzzlehttp/promises" package is not installed. Try running "composer require guzzlehttp/promises".', __METHOD__));
        }
        try {
            $response = $this->sendPsr7Request($request, \true);
        } catch (\ZOOlanders\YOOessentials\Vendor\Http\Client\Exception\NetworkException $e) {
            return new \ZOOlanders\YOOessentials\Vendor\Symfony\Component\HttpClient\Response\HttplugPromise(new \ZOOlanders\YOOessentials\Vendor\GuzzleHttp\Promise\RejectedPromise($e));
        }
        $waitLoop = $this->waitLoop;
        $promise = new \ZOOlanders\YOOessentials\Vendor\GuzzleHttp\Promise\Promise(static function () use($response, $waitLoop) {
            $waitLoop->wait($response);
        }, static function () use($response, $promisePool) {
            $response->cancel();
            unset($promisePool[$response]);
        });
        $promisePool[$response] = [$request, $promise];
        return new \ZOOlanders\YOOessentials\Vendor\Symfony\Component\HttpClient\Response\HttplugPromise($promise);
    }
    /**
     * Resolves pending promises that complete before the timeouts are reached.
     *
     * When $maxDuration is null and $idleTimeout is reached, promises are rejected.
     *
     * @return int The number of remaining pending promises
     */
    public function wait(float $maxDuration = null, float $idleTimeout = null) : int
    {
        return $this->waitLoop->wait(null, $maxDuration, $idleTimeout);
    }
    /**
     * {@inheritdoc}
     */
    public function createRequest($method, $uri, array $headers = [], $body = null, $protocolVersion = '1.1') : \ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\RequestInterface
    {
        if ($this->responseFactory instanceof \ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\RequestFactoryInterface) {
            $request = $this->responseFactory->createRequest($method, $uri);
        } elseif (\class_exists(\ZOOlanders\YOOessentials\Vendor\Nyholm\Psr7\Request::class)) {
            $request = new \ZOOlanders\YOOessentials\Vendor\Nyholm\Psr7\Request($method, $uri);
        } elseif (\class_exists(\ZOOlanders\YOOessentials\Vendor\Http\Discovery\Psr17FactoryDiscovery::class)) {
            $request = \ZOOlanders\YOOessentials\Vendor\Http\Discovery\Psr17FactoryDiscovery::findRequestFactory()->createRequest($method, $uri);
        } else {
            throw new \LogicException(\sprintf('You cannot use "%s()" as the "nyholm/psr7" package is not installed. Try running "composer require nyholm/psr7".', __METHOD__));
        }
        $request = $request->withProtocolVersion($protocolVersion)->withBody($this->createStream($body));
        foreach ($headers as $name => $value) {
            $request = $request->withAddedHeader($name, $value);
        }
        return $request;
    }
    /**
     * {@inheritdoc}
     */
    public function createStream($body = null) : \ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\StreamInterface
    {
        if ($body instanceof \ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\StreamInterface) {
            return $body;
        }
        if (\is_string($body ?? '')) {
            $stream = $this->streamFactory->createStream($body ?? '');
        } elseif (\is_resource($body)) {
            $stream = $this->streamFactory->createStreamFromResource($body);
        } else {
            throw new \InvalidArgumentException(\sprintf('"%s()" expects string, resource or StreamInterface, "%s" given.', __METHOD__, \get_debug_type($body)));
        }
        if ($stream->isSeekable()) {
            $stream->seek(0);
        }
        return $stream;
    }
    /**
     * {@inheritdoc}
     */
    public function createUri($uri) : \ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\UriInterface
    {
        if ($uri instanceof \ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\UriInterface) {
            return $uri;
        }
        if ($this->responseFactory instanceof \ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\UriFactoryInterface) {
            return $this->responseFactory->createUri($uri);
        }
        if (\class_exists(\ZOOlanders\YOOessentials\Vendor\Nyholm\Psr7\Uri::class)) {
            return new \ZOOlanders\YOOessentials\Vendor\Nyholm\Psr7\Uri($uri);
        }
        if (\class_exists(\ZOOlanders\YOOessentials\Vendor\Http\Discovery\Psr17FactoryDiscovery::class)) {
            return \ZOOlanders\YOOessentials\Vendor\Http\Discovery\Psr17FactoryDiscovery::findUrlFactory()->createUri($uri);
        }
        throw new \LogicException(\sprintf('You cannot use "%s()" as the "nyholm/psr7" package is not installed. Try running "composer require nyholm/psr7".', __METHOD__));
    }
    public function __sleep() : array
    {
        throw new \BadMethodCallException('Cannot serialize ' . __CLASS__);
    }
    public function __wakeup()
    {
        throw new \BadMethodCallException('Cannot unserialize ' . __CLASS__);
    }
    public function __destruct()
    {
        $this->wait();
    }
    public function reset()
    {
        if ($this->client instanceof \ZOOlanders\YOOessentials\Vendor\Symfony\Contracts\Service\ResetInterface) {
            $this->client->reset();
        }
    }
    private function sendPsr7Request(\ZOOlanders\YOOessentials\Vendor\Psr\Http\Message\RequestInterface $request, bool $buffer = null) : \ZOOlanders\YOOessentials\Vendor\Symfony\Contracts\HttpClient\ResponseInterface
    {
        try {
            $body = $request->getBody();
            if ($body->isSeekable()) {
                $body->seek(0);
            }
            return $this->client->request($request->getMethod(), (string) $request->getUri(), ['headers' => $request->getHeaders(), 'body' => $body->getContents(), 'http_version' => '1.0' === $request->getProtocolVersion() ? '1.0' : null, 'buffer' => $buffer]);
        } catch (\InvalidArgumentException $e) {
            throw new \ZOOlanders\YOOessentials\Vendor\Http\Client\Exception\RequestException($e->getMessage(), $request, $e);
        } catch (\ZOOlanders\YOOessentials\Vendor\Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface $e) {
            throw new \ZOOlanders\YOOessentials\Vendor\Http\Client\Exception\NetworkException($e->getMessage(), $request, $e);
        }
    }
}
